#[cfg_attr(feature = "uniffi", derive(uniffi::Enum))]
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(u8)]
pub enum Parsha {
    NONE = 0,
    BERESHIS = 1,
    NOACH = 2,
    #[allow(non_camel_case_types)]
    LECH_LECHA = 3,
    VAYERA = 4,
    #[allow(non_camel_case_types)]
    CHAYEI_SARA = 5,
    TOLDOS = 6,
    VAYETZEI = 7,
    VAYISHLACH = 8,
    VAYESHEV = 9,
    MIKETZ = 10,
    VAYIGASH = 11,
    VAYECHI = 12,
    SHEMOS = 13,
    VAERA = 14,
    BO = 15,
    BESHALACH = 16,
    YISRO = 17,
    MISHPATIM = 18,
    TERUMAH = 19,
    TETZAVEH = 20,
    #[allow(non_camel_case_types)]
    KI_SISA = 21,
    VAYAKHEL = 22,
    PEKUDEI = 23,
    VAYIKRA = 24,
    TZAV = 25,
    SHMINI = 26,
    TAZRIA = 27,
    METZORA = 28,
    #[allow(non_camel_case_types)]
    ACHREI_MOS = 29,
    KEDOSHIM = 30,
    EMOR = 31,
    BEHAR = 32,
    BECHUKOSAI = 33,
    BAMIDBAR = 34,
    NASSO = 35,
    BEHAALOSCHA = 36,
    SHLACH = 37,
    KORACH = 38,
    CHUKAS = 39,
    BALAK = 40,
    PINCHAS = 41,
    MATOS = 42,
    MASEI = 43,
    DEVARIM = 44,
    VAESCHANAN = 45,
    EIKEV = 46,
    REEH = 47,
    SHOFTIM = 48,
    #[allow(non_camel_case_types)]
    KI_SEITZEI = 49,
    #[allow(non_camel_case_types)]
    KI_SAVO = 50,
    #[allow(non_camel_case_types)]
    NITZAVIM = 51,
    #[allow(non_camel_case_types)]
    VAYEILECH = 52,
    HAAZINU = 53,
    #[allow(non_camel_case_types)]
    VZOS_HABERACHA = 54,
    #[allow(non_camel_case_types)]
    VAYAKHEL_PEKUDEI = 55,
    #[allow(non_camel_case_types)]
    TAZRIA_METZORA = 56,
    #[allow(non_camel_case_types)]
    ACHREI_MOS_KEDOSHIM = 57,
    #[allow(non_camel_case_types)]
    BEHAR_BECHUKOSAI = 58,
    #[allow(non_camel_case_types)]
    CHUKAS_BALAK = 59,
    #[allow(non_camel_case_types)]
    MATOS_MASEI = 60,
    #[allow(non_camel_case_types)]
    NITZAVIM_VAYEILECH = 61,
    SHKALIM = 62,
    ZACHOR = 63,
    PARA = 64,
    HACHODESH = 65,
    SHUVA = 66,
    SHIRA = 67,
    HAGADOL = 68,
    CHAZON = 69,
    NACHAMU = 70,
}

pub(crate) const PARSHA_LIST_0: [Parsha; 51] = [
    Parsha::NONE,
    Parsha::VAYEILECH,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL_PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::NONE,
    Parsha::SHMINI,
    Parsha::TAZRIA_METZORA,
    Parsha::ACHREI_MOS_KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR_BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
pub(crate) const PARSHA_LIST_1: [Parsha; 51] = [
    Parsha::NONE,
    Parsha::VAYEILECH,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL_PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::NONE,
    Parsha::SHMINI,
    Parsha::TAZRIA_METZORA,
    Parsha::ACHREI_MOS_KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR_BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NONE,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS_BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
pub(crate) const PARSHA_LIST_2: [Parsha; 52] = [
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL_PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::SHMINI,
    Parsha::TAZRIA_METZORA,
    Parsha::ACHREI_MOS_KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR_BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM,
];
pub(crate) const PARSHA_LIST_3: [Parsha; 52] = [
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::NONE,
    Parsha::SHMINI,
    Parsha::TAZRIA_METZORA,
    Parsha::ACHREI_MOS_KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR_BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM,
];
pub(crate) const PARSHA_LIST_4: [Parsha; 52] = [
    Parsha::NONE,
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL_PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::NONE,
    Parsha::SHMINI,
    Parsha::TAZRIA_METZORA,
    Parsha::ACHREI_MOS_KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR_BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM,
];
pub(crate) const PARSHA_LIST_5: [Parsha; 52] = [
    Parsha::NONE,
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL_PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::NONE,
    Parsha::SHMINI,
    Parsha::TAZRIA_METZORA,
    Parsha::ACHREI_MOS_KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR_BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
pub(crate) const PARSHA_LIST_6: [Parsha; 55] = [
    Parsha::NONE,
    Parsha::VAYEILECH,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::SHMINI,
    Parsha::TAZRIA,
    Parsha::METZORA,
    Parsha::NONE,
    Parsha::ACHREI_MOS,
    Parsha::KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NONE,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS_BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
pub const PARSHA_LIST_7: [Parsha; 56] = [
    Parsha::NONE,
    Parsha::VAYEILECH,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::SHMINI,
    Parsha::TAZRIA,
    Parsha::METZORA,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::ACHREI_MOS,
    Parsha::KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM,
];
pub(crate) const PARSHA_LIST_8: [Parsha; 56] = [
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::SHMINI,
    Parsha::TAZRIA,
    Parsha::METZORA,
    Parsha::ACHREI_MOS,
    Parsha::NONE,
    Parsha::KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS,
    Parsha::MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM,
];
pub(crate) const PARSHA_LIST_9: [Parsha; 56] = [
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::SHMINI,
    Parsha::TAZRIA,
    Parsha::METZORA,
    Parsha::ACHREI_MOS,
    Parsha::NONE,
    Parsha::KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS,
    Parsha::MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
pub(crate) const PARSHA_LIST_10: [Parsha; 56] = [
    Parsha::NONE,
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::SHMINI,
    Parsha::TAZRIA,
    Parsha::METZORA,
    Parsha::NONE,
    Parsha::ACHREI_MOS,
    Parsha::KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
pub(crate) const PARSHA_LIST_11: [Parsha; 56] = [
    Parsha::NONE,
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::SHMINI,
    Parsha::TAZRIA,
    Parsha::METZORA,
    Parsha::NONE,
    Parsha::ACHREI_MOS,
    Parsha::KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NONE,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS_BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
pub(crate) const PARSHA_LIST_12: [Parsha; 51] = [
    Parsha::NONE,
    Parsha::VAYEILECH,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL_PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::NONE,
    Parsha::SHMINI,
    Parsha::TAZRIA_METZORA,
    Parsha::ACHREI_MOS_KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR_BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
pub(crate) const PARSHA_LIST_13: [Parsha; 52] = [
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL_PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::NONE,
    Parsha::SHMINI,
    Parsha::TAZRIA_METZORA,
    Parsha::ACHREI_MOS_KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM,
];
pub(crate) const PARSHA_LIST_14: [Parsha; 55] = [
    Parsha::NONE,
    Parsha::VAYEILECH,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::SHMINI,
    Parsha::TAZRIA,
    Parsha::METZORA,
    Parsha::NONE,
    Parsha::ACHREI_MOS,
    Parsha::KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
pub(crate) const PARSHA_LIST_15: [Parsha; 56] = [
    Parsha::NONE,
    Parsha::VAYEILECH,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::SHMINI,
    Parsha::TAZRIA,
    Parsha::METZORA,
    Parsha::NONE,
    Parsha::ACHREI_MOS,
    Parsha::KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS,
    Parsha::MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM,
];
pub(crate) const PARSHA_LIST_16: [Parsha; 56] = [
    Parsha::NONE,
    Parsha::NONE,
    Parsha::HAAZINU,
    Parsha::NONE,
    Parsha::NONE,
    Parsha::BERESHIS,
    Parsha::NOACH,
    Parsha::LECH_LECHA,
    Parsha::VAYERA,
    Parsha::CHAYEI_SARA,
    Parsha::TOLDOS,
    Parsha::VAYETZEI,
    Parsha::VAYISHLACH,
    Parsha::VAYESHEV,
    Parsha::MIKETZ,
    Parsha::VAYIGASH,
    Parsha::VAYECHI,
    Parsha::SHEMOS,
    Parsha::VAERA,
    Parsha::BO,
    Parsha::BESHALACH,
    Parsha::YISRO,
    Parsha::MISHPATIM,
    Parsha::TERUMAH,
    Parsha::TETZAVEH,
    Parsha::KI_SISA,
    Parsha::VAYAKHEL,
    Parsha::PEKUDEI,
    Parsha::VAYIKRA,
    Parsha::TZAV,
    Parsha::SHMINI,
    Parsha::TAZRIA,
    Parsha::METZORA,
    Parsha::NONE,
    Parsha::ACHREI_MOS,
    Parsha::KEDOSHIM,
    Parsha::EMOR,
    Parsha::BEHAR,
    Parsha::BECHUKOSAI,
    Parsha::BAMIDBAR,
    Parsha::NASSO,
    Parsha::BEHAALOSCHA,
    Parsha::SHLACH,
    Parsha::KORACH,
    Parsha::CHUKAS,
    Parsha::BALAK,
    Parsha::PINCHAS,
    Parsha::MATOS_MASEI,
    Parsha::DEVARIM,
    Parsha::VAESCHANAN,
    Parsha::EIKEV,
    Parsha::REEH,
    Parsha::SHOFTIM,
    Parsha::KI_SEITZEI,
    Parsha::KI_SAVO,
    Parsha::NITZAVIM_VAYEILECH,
];
